<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 01/21/2013 *
 * ===================                          Last change: 01/21/2013 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : blacklist_functions.php                          *
 * -------------------------------------------------------------------- *
 * Short description : Functions for ext-                               *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer ext-                             *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Checks whether given email is blacklisted
function isEmailBlacklisted ($email) {
	// Call inner function
	return isGenericBlacklisted('email', $email);
}

// Checks whether given IP is blacklisted
function isIpBlacklisted ($ip) {
	// Call inner function
	return isGenericBlacklisted('ip', $ip);
}

// Checks whether given URL is blacklisted
function isUrlBlacklisted ($url) {
	// Call inner function
	return isGenericBlacklisted('url', $email);
}

// Checks whether given data is blacklisted
function isGenericBlacklisted ($type, $data) {
	// Mark it as not listed by default
	$listed = FALSE;

	// Is black-listing enbaled?
	if (!isGenericBlacklistEnabled($type)) {
		// No, then all emails are not in this list
		return FALSE;
	} elseif (!isset($GLOBALS['blacklist_data'][$type][$data])) {
		// Check black-list for given email
		$result = sqlQueryEscaped("SELECT
	`id`,
	`data`,
	`pool_id`,
	`provider`,
	`type`,
	UNIX_TIMESTAMP(`added`) AS `added`
FROM
	`{?_MYSQL_PREFIX?}_blacklist`
WHERE
	'%s' REGEXP `data` AND
	`type`='%s'
LIMIT 1",
			array(
				$data,
				strtoupper($type)
			), __FUNCTION__, __LINE__);

		// Is there an entry?
		if (sqlNumRows($result) == 1) {
			// Jupp, we got one listed
			$GLOBALS['blacklist_data'][$type][$data] = sqlFetchArray($result);

			// Mark it as listed
			$listed = TRUE;
		} // END - if

		// Free result
		sqlFreeResult($result);
	} else {
		// Is found in cache -> black-listed
		$listed = TRUE;
	}

	// Return result
	return $listed;
}

// Inserts a given email (pattern) in blacklist if not found
function insertEmailInBlacklist ($email, $provider = 'BLACKLIST') {
	// Call inner function
	return insertGenericInBlacklist ('email', $email, NULL, $provider);
}

// Inserts a given IP (pattern) in blacklist if not found
function insertIpInBlacklist ($ip, $provider = 'BLACKLIST') {
	// Call inner function
	return insertGenericInBlacklist ('ip', $ip, NULL, $provider);
}

// Inserts a given URL (pattern) in blacklist if not found
function insertUrlInBlacklist ($url, $poolId, $provider = 'BLACKLIST') {
	// Call inner function
	return insertGenericInBlacklist ('url', $url, $poolId, $provider);
}

// Inserts a given URL in blacklist if not found
function insertGenericInBlacklist ($type, $data, $poolId = NULL, $provider = 'BLACKLIST') {
	// Is this feature turned on and is the URL not there?
	if (!isGenericBlacklistEnabled($type)) {
		// Not enabled, then please don't call this function
		reportBug(__FUNCTION__, __LINE__, 'Blacklisting of type ' . $type . ' is disabled, data=' . $data . ',poolId=' . convertZeroToNull($poolId));
	} elseif (!isUrlBlacklisted($data)) {
		// Did not find a record so we can add it... :)
		sqlQueryEscaped("INSERT INTO
	`{?_MYSQL_PREFIX?}_blacklist`
(
	`data`,
	`pool_id`,
	`provider`,
	`type`
) VALUES (
	'%s',
	%s,
	'%s',
	'%s'
)",
		array(
			$data,
			convertZeroToNull($poolId),
			$provider,
			strtoupper($type)
		), __FUNCTION__, __LINE__);
	} // END - if

	// Return insert id for debugging/reporting pursposes
	return getSqlInsertId();
}

// ----------------------------------------------------------------------------
//                      Configuration wrapper functions
// ----------------------------------------------------------------------------

// Generic wrapper
function isGenericBlacklistEnabled ($type) {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig($type . '_blacklist') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for url_blacklist
function getUrlBlacklist () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('url_blacklist');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Wrapper to check if url_blacklist is enabled
function isUrlBlacklistEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getUrlBlacklist() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for email_blacklist
function getEmailBlacklist () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('email_blacklist');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Wrapper to check if email_blacklist is enabled
function isEmailBlacklistEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getEmailBlacklist() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for ip_blacklist
function getIpBlacklist () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ip_blacklist');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Wrapper to check if ip_blacklist is enabled
function isIpBlacklistEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getIpBlacklist() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// [EOF]
?>
